@@ -16,7 +16,8 @@
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+   02110-1301 USA
 
 */
 
@@ -92,10 +93,12 @@
  * \function igraph_vector_init
  * \brief Initializes a vector object (constructor).
  * 
+ * </para><para>
  * Every vector needs to be initialized before it can be used, and
  * there are a number of initialization functions or otherwise called
  * constructors. 
  * 
+ * </para><para>
  * Every vector object initialized by this function should be
  * destroyed (ie. the memory allocated for it should be freed) when it
  * is not needed anymore, the \ref igraph_vector_destroy() function is
@@ -114,7 +117,7 @@
 int igraph_vector_init      (igraph_vector_t* v, int long size) {	
         long int alloc_size= size > 0 ? size : 1;
 	if (size < 0) { size=0; }
-	v->stor_begin=Calloc(alloc_size, real_t);
+	v->stor_begin=Calloc(alloc_size, igraph_real_t);
 	if (v->stor_begin==0) {
 	  IGRAPH_ERROR("cannot init vector", IGRAPH_ENOMEM);
 	}
@@ -129,6 +132,7 @@ int igraph_vector_init      (igraph_vector_t* v, int long size) {
  * \function igraph_vector_view
  * \brief Handle a regular C array as a \type igraph_vector_t.
  * 
+ * </para><para>
  * This is a special \type igraph_vector_t constructor. It allows to
  * handle a regular C array as a \type igraph_vector_t temporarily.
  * Be sure that you \em don't ever call the destructor (\ref
@@ -142,11 +146,11 @@ int igraph_vector_init      (igraph_vector_t* v, int long size) {
  * Time complexity: O(1)
  */ 
 
-const igraph_vector_t *igraph_vector_view (const igraph_vector_t *v, const real_t *data, 
+const igraph_vector_t *igraph_vector_view (const igraph_vector_t *v, const igraph_real_t *data, 
 			     long int length) {
   igraph_vector_t *v2=(igraph_vector_t*) v;
-  v2->stor_begin=(real_t*)data;
-  v2->stor_end=(real_t*)data+length;
+  v2->stor_begin=(igraph_real_t*)data;
+  v2->stor_end=(igraph_real_t*)data+length;
   v2->end=v2->stor_end;
   return v;
 }
@@ -156,6 +160,7 @@ const igraph_vector_t *igraph_vector_view (const igraph_vector_t *v, const real_
  * \function igraph_vector_init_real
  * \brief Create an \type igraph_vector_t from the parameters.
  * 
+ * </para><para>
  * Because of how C and the C library handles variable length argument
  * lists, it is required that you supply real constants to this
  * function. This means that
@@ -166,7 +171,7 @@ const igraph_vector_t *igraph_vector_view (const igraph_vector_t *v, const real_
  * \verbatim igraph_vector_t v;
  * igraph_vector_init_real(&amp;v, 5, 1.0,2.0,3.0,4.0,5.0); \endverbatim
  * \param v Pointer to an uninitialized \type igraph_vector_t object.
- * \param no Positive integer, the number of \type real_t
+ * \param no Positive integer, the number of \type igraph_real_t
  *    parameters to follow.
  * \param ... The elements of the vector.
  * \return Error code, this can be \c IGRAPH_ENOMEM
@@ -187,7 +192,7 @@ int igraph_vector_init_real(igraph_vector_t *v, int no, ...) {
 
   va_start(ap, no);
   for (i=0; i<no; i++) {
-    VECTOR(*v)[i]=(real_t) va_arg(ap, double);
+    VECTOR(*v)[i]=(igraph_real_t) va_arg(ap, double);
   }
   va_end(ap);
   
@@ -199,6 +204,7 @@ int igraph_vector_init_real(igraph_vector_t *v, int no, ...) {
  * \function igraph_vector_init_real_end
  * \brief Create an \type igraph_vector_t from the parameters.
  * 
+ * </para><para>
  * This constructor is similar to \ref igraph_vector_init_real(), the only
  * difference is that instead of giving the number of elements in the
  * vector, a special marker element follows the last real vector
@@ -218,13 +224,13 @@ int igraph_vector_init_real(igraph_vector_t *v, int no, ...) {
  * complexity of the memory allocation.
  */
 
-int igraph_vector_init_real_end(igraph_vector_t *v, real_t endmark, ...) {
+int igraph_vector_init_real_end(igraph_vector_t *v, igraph_real_t endmark, ...) {
   int i=0, n=0;
   va_list ap;
 
   va_start(ap, endmark);
   while (1) {
-    real_t num = va_arg(ap, double);
+    igraph_real_t num = va_arg(ap, double);
     if (num == endmark) {
       break;
     }
@@ -236,7 +242,7 @@ int igraph_vector_init_real_end(igraph_vector_t *v, real_t endmark, ...) {
   
   va_start(ap, endmark);
   for (i=0; i<n; i++) {
-    VECTOR(*v)[i]=(real_t) va_arg(ap, double);
+    VECTOR(*v)[i]=(igraph_real_t) va_arg(ap, double);
   }
   va_end(ap);
   
@@ -249,6 +255,7 @@ int igraph_vector_init_real_end(igraph_vector_t *v, real_t endmark, ...) {
  * \function igraph_vector_init_int
  * \brief Create an \type igraph_vector_t containing the parameters.
  * 
+ * </para><para>
  * This function is similar to \ref igraph_vector_init_real(), but it expects 
  * \type int parameters. It is important that all parameters
  * should be of this type, otherwise the result of the function call
@@ -273,7 +280,7 @@ int igraph_vector_init_int(igraph_vector_t *v, int no, ...) {
 
   va_start(ap, no);
   for (i=0; i<no; i++) {
-    VECTOR(*v)[i]=(real_t) va_arg(ap, int);
+    VECTOR(*v)[i]=(igraph_real_t) va_arg(ap, int);
   }
   va_end(ap);
   
@@ -285,6 +292,7 @@ int igraph_vector_init_int(igraph_vector_t *v, int no, ...) {
  * \function igraph_vector_init_int_end
  * \brief Create an \type igraph_vector_t from the parameters.
  * 
+ * </para><para>
  * This constructor is similar to \ref igraph_vector_init_int(), the only
  * difference is that instead of giving the number of elements in the
  * vector, a special marker element follows the last real vector
@@ -322,7 +330,7 @@ int igraph_vector_init_int_end(igraph_vector_t *v, int endmark, ...) {
   
   va_start(ap, endmark);
   for (i=0; i<n; i++) {
-    VECTOR(*v)[i]=(real_t) va_arg(ap, int);
+    VECTOR(*v)[i]=(igraph_real_t) va_arg(ap, int);
   }
   va_end(ap);
   
@@ -335,6 +343,7 @@ int igraph_vector_init_int_end(igraph_vector_t *v, int endmark, ...) {
  * \function igraph_vector_destroy
  * \brief Destroys a vector object.
  *
+ * </para><para>
  * All vectors initialized by \ref igraph_vector_init() should be properly
  * destroyed by this function. A destroyed vector needs to be
  * reinitialized by \ref igraph_vector_init(), \ref igraph_vector_init_copy() or
@@ -358,12 +367,14 @@ void igraph_vector_destroy   (igraph_vector_t* v) {
  * \function igraph_vector_reserve
  * \brief Reserves memory for a vector.
  * 
+ * </para><para>
  * \a igraph vectors are flexible, they can grow and
  * shrink. Growing 
  * however occasionally needs the data in the vector to be copyed.
  * In order to avoid you can call this function to reserve space for
  * future growth of the vector. 
  * 
+ * </para><para>
  * Note that this function does \em not change the size of the
  * vector. Let us see a small example to clarify things: if you
  * reserve space for 100 elements and the size of your
@@ -381,12 +392,12 @@ void igraph_vector_destroy   (igraph_vector_t* v) {
 
 int igraph_vector_reserve   (igraph_vector_t* v, long int size) {
 	long int actual_size=igraph_vector_size(v);
-	real_t *tmp;
+	igraph_real_t *tmp;
 	assert(v != NULL);
 	assert(v->stor_begin != NULL);
 	if (size <= igraph_vector_size(v)) { return 0; }
 
-	tmp=Realloc(v->stor_begin, size, real_t);
+	tmp=Realloc(v->stor_begin, size, igraph_real_t);
 	if (tmp==0) {
 	  IGRAPH_ERROR("cannot reserve space for vector", IGRAPH_ENOMEM);
 	}
@@ -409,7 +420,7 @@ int igraph_vector_reserve   (igraph_vector_t* v, long int size) {
  * Time complexity: O(1).
  */
 
-bool_t igraph_vector_empty     (const igraph_vector_t* v) {
+igraph_bool_t igraph_vector_empty     (const igraph_vector_t* v) {
 	assert(v != NULL);
 	assert(v->stor_begin != NULL);
 	return v->stor_begin == v->end;
@@ -437,6 +448,7 @@ long int igraph_vector_size      (const igraph_vector_t* v) {
  * \function igraph_vector_clear
  * \brief Removes all elements from a vector.
  * 
+ * </para><para>
  * This function simply sets the size of the vector to zero, it does
  * not free any allocated memory. For that you have to call
  * \ref igraph_vector_destroy().
@@ -456,6 +468,7 @@ void igraph_vector_clear     (igraph_vector_t* v) {
  * \function igraph_vector_push_back
  * \brief Appends one element to a vector.
  * 
+ * </para><para>
  * This function resizes the vector to be one element longer and
  * sets the very last element in the vector to \p e.
  * \param v The vector object.
@@ -475,7 +488,7 @@ void igraph_vector_clear     (igraph_vector_t* v) {
  * complexity of memory allocation is at most linear.)
  */
 
-int igraph_vector_push_back (igraph_vector_t* v, real_t e) {
+int igraph_vector_push_back (igraph_vector_t* v, igraph_real_t e) {
   	assert(v != NULL);
 	assert(v->stor_begin != NULL);
 	
@@ -504,7 +517,7 @@ int igraph_vector_push_back (igraph_vector_t* v, real_t e) {
  * vector. \ref igraph_vector_e_ptr() returns the address of an element.</para>
  * 
  * <para>\ref igraph_vector_tail() returns the last element of a non-empty
- * vector. There is no <function>igraph_vector_tail()</function> function
+ * vector. There is no <function>igraph_vector_head()</function> function
  * however, as it is easy to write <code>VECTOR(v)[0]</code>
  * instead.</para>
  */
@@ -522,7 +535,7 @@ int igraph_vector_push_back (igraph_vector_t* v, real_t e) {
  * Time complexity: O(1).
  */
 
-real_t igraph_vector_e         (const igraph_vector_t* v, long int pos) {
+igraph_real_t igraph_vector_e         (const igraph_vector_t* v, long int pos) {
 	assert(v != NULL);
 	assert(v->stor_begin != NULL);
 	return * (v->stor_begin + pos);
@@ -541,7 +554,7 @@ real_t igraph_vector_e         (const igraph_vector_t* v, long int pos) {
  * Time complexity: O(1).
  */
 
-real_t*igraph_vector_e_ptr  (const igraph_vector_t* v, long int pos) {
+igraph_real_t*igraph_vector_e_ptr  (const igraph_vector_t* v, long int pos) {
   assert(v!=NULL);
   assert(v->stor_begin != NULL);
   return v->stor_begin+pos;
@@ -557,7 +570,7 @@ real_t*igraph_vector_e_ptr  (const igraph_vector_t* v, long int pos) {
  * \sa \ref igraph_vector_e().
  */
 
-void igraph_vector_set       (igraph_vector_t* v, long int pos, real_t value) {
+void igraph_vector_set       (igraph_vector_t* v, long int pos, igraph_real_t value) {
 	assert(v != NULL);
 	assert(v->stor_begin != NULL);	
 	*(v->stor_begin + pos) = value;
@@ -568,6 +581,7 @@ void igraph_vector_set       (igraph_vector_t* v, long int pos, real_t value) {
  * \function igraph_vector_null
  * \brief Sets each element in the vector to zero.
  * 
+ * </para><para>
  * Note that \ref igraph_vector_init() sets the elements to zero as well, so
  * it makes no sense to call this function on a just initialized
  * vector. 
@@ -581,7 +595,7 @@ void igraph_vector_null      (igraph_vector_t* v) {
 	assert(v != NULL);
 	assert(v->stor_begin != NULL);
 	if (igraph_vector_size(v)>0) {
-		memset(v->stor_begin, 0, sizeof(real_t)*igraph_vector_size(v));
+		memset(v->stor_begin, 0, sizeof(igraph_real_t)*igraph_vector_size(v));
 	}
 }
 
@@ -590,6 +604,7 @@ void igraph_vector_null      (igraph_vector_t* v) {
  * \function igraph_vector_tail
  * \brief Returns the last element in a vector.
  *
+ * </para><para>
  * It is an error to call this function on an empty vector, the result
  * is undefined.
  * \param v The vector object.
@@ -598,7 +613,7 @@ void igraph_vector_null      (igraph_vector_t* v) {
  * Time complexity: O(1).
  */
 
-real_t igraph_vector_tail(const igraph_vector_t *v) {
+igraph_real_t igraph_vector_tail(const igraph_vector_t *v) {
   assert(v!=NULL);
   assert(v->stor_begin != NULL);
   return *((v->end)-1);
@@ -609,6 +624,7 @@ real_t igraph_vector_tail(const igraph_vector_t *v) {
  * \function igraph_vector_pop_back
  * \brief Removes and returns the last element of a vector.
  *
+ * </para><para>
  * It is an error to call this function with an empty vector.
  * \param v The vector object.
  * \return The removed last element.
@@ -616,8 +632,8 @@ real_t igraph_vector_tail(const igraph_vector_t *v) {
  * Time complexity: O(1).
  */
 
-real_t igraph_vector_pop_back(igraph_vector_t* v) {
-  real_t tmp;
+igraph_real_t igraph_vector_pop_back(igraph_vector_t* v) {
+  igraph_real_t tmp;
   assert(v!=NULL);
   assert(v->stor_begin != NULL);
   assert(v->end != v->stor_begin);
@@ -631,6 +647,7 @@ real_t igraph_vector_pop_back(igraph_vector_t* v) {
  * \function igraph_vector_order
  * \brief Calculate the order of the elements in a vector.
  *
+ * </para><para>
  * The smallest element will have order zero, the second smallest
  * order one, etc. 
  * \param v The original \type igraph_vector_t object.
@@ -644,7 +661,7 @@ real_t igraph_vector_pop_back(igraph_vector_t* v) {
  * Time complexity: O()
  */
 
-int igraph_vector_order(const igraph_vector_t* v, igraph_vector_t* res, integer_t nodes) {
+int igraph_vector_order(const igraph_vector_t* v, igraph_vector_t* res, igraph_integer_t nodes) {
   long int edges=igraph_vector_size(v);
   igraph_vector_t ptr;
   igraph_vector_t rad;
@@ -684,6 +701,25 @@ int igraph_vector_order(const igraph_vector_t* v, igraph_vector_t* res, integer_
   return 0;
 }
 
+int igraph_vector_order2(igraph_vector_t *v) {
+
+  igraph_indheap_t heap;
+
+  igraph_indheap_init_array(&heap, VECTOR(*v), igraph_vector_size(v));
+  IGRAPH_FINALLY(igraph_indheap_destroy, &heap);
+
+  igraph_vector_clear(v);
+  while (!igraph_indheap_empty(&heap)) {
+    IGRAPH_CHECK(igraph_vector_push_back(v, 
+					 igraph_indheap_max_index(&heap)-1));
+    igraph_indheap_delete_max(&heap);
+  }
+  
+  igraph_indheap_destroy(&heap);
+  IGRAPH_FINALLY_CLEAN(1);
+  return 0;
+}
+
 /**
  * \ingroup vector
  * \function igraph_vector_sort_cmp
@@ -692,8 +728,8 @@ int igraph_vector_order(const igraph_vector_t* v, igraph_vector_t* res, integer_
  */
 
 int igraph_vector_sort_cmp(const void *a, const void *b) {
-  const real_t *da = (const real_t *) a;
-  const real_t *db = (const real_t *) b;
+  const igraph_real_t *da = (const igraph_real_t *) a;
+  const igraph_real_t *db = (const igraph_real_t *) b;
 
   return (*da > *db) - (*da < *db);
 }
@@ -703,6 +739,7 @@ int igraph_vector_sort_cmp(const void *a, const void *b) {
  * \function igraph_vector_sort
  * \brief Sorts the elements of the vector into ascending order.
  * 
+ * </para><para>
  * This function uses the built-in sort function of the C library.
  * \param v Pointer to an initialized vector object.
  *
@@ -715,7 +752,7 @@ int igraph_vector_sort_cmp(const void *a, const void *b) {
 void igraph_vector_sort(igraph_vector_t *v) {
   assert(v != NULL);
   assert(v->stor_begin != NULL);
-  qsort(v->stor_begin, igraph_vector_size(v), sizeof(real_t), igraph_vector_sort_cmp);
+  qsort(v->stor_begin, igraph_vector_size(v), sizeof(igraph_real_t), igraph_vector_sort_cmp);
 }
 
 /**
@@ -723,6 +760,7 @@ void igraph_vector_sort(igraph_vector_t *v) {
  * \function igraph_vector_resize
  * \brief Resize the vector.
  *
+ * </para><para>
  * Note that this function does not free any memory, just sets the
  * size of the vector to the given one. It can on the other hand 
  * allocate more memory if the new size is larger than the previous
@@ -757,6 +795,7 @@ int igraph_vector_resize(igraph_vector_t* v, long int newsize) {
  * \function igraph_vector_max
  * \brief Gives the maximum element of the vector.
  *
+ * </para><para>
  * If the size of the vector is zero, an arbitrary number is
  * returned.
  * \param v The vector object.
@@ -766,9 +805,9 @@ int igraph_vector_resize(igraph_vector_t* v, long int newsize) {
  * n is the size of the vector. 
  */
 
-real_t igraph_vector_max(const igraph_vector_t* v) {
-  real_t max;
-  real_t *ptr;
+igraph_real_t igraph_vector_max(const igraph_vector_t* v) {
+  igraph_real_t max;
+  igraph_real_t *ptr;
   assert(v != NULL);
   assert(v->stor_begin != NULL);
   max=*(v->stor_begin);
@@ -787,6 +826,7 @@ real_t igraph_vector_max(const igraph_vector_t* v) {
  * \function igraph_vector_which_max
  * \brief Gives the position of the maximum element of the vector.
  *
+ * </para><para>
  * If the size of the vector is zero, -1 is 
  * returned.
  * \param v The vector object.
@@ -799,8 +839,8 @@ real_t igraph_vector_max(const igraph_vector_t* v) {
 long int igraph_vector_which_max(const igraph_vector_t* v) {
   long int which=-1;
   if (!igraph_vector_empty(v)) {
-    real_t max;
-    real_t *ptr;
+    igraph_real_t max;
+    igraph_real_t *ptr;
     long int pos;
     assert(v != NULL);
     assert(v->stor_begin != NULL);
@@ -832,14 +872,14 @@ long int igraph_vector_which_max(const igraph_vector_t* v) {
  * O(\p length).
  */
 
-int igraph_vector_init_copy(igraph_vector_t *v, real_t *data, long int length) {
-  v->stor_begin=Calloc(length, real_t);
+int igraph_vector_init_copy(igraph_vector_t *v, igraph_real_t *data, long int length) {
+  v->stor_begin=Calloc(length, igraph_real_t);
   if (v->stor_begin==0) {
     IGRAPH_ERROR("cannot init vector from array", IGRAPH_ENOMEM);
   }
   v->stor_end=v->stor_begin+length;
   v->end=v->stor_end;
-  memcpy(v->stor_begin, data, length*sizeof(real_t));
+  memcpy(v->stor_begin, data, length*sizeof(igraph_real_t));
   
   return 0;
 }
@@ -849,6 +889,7 @@ int igraph_vector_init_copy(igraph_vector_t *v, real_t *data, long int length) {
  * \function igraph_vector_copy_to
  * \brief Copies the contents of a vector to a C array.
  * 
+ * </para><para>
  * The C array should have sufficient length.
  * \param v The vector object.
  * \param to The C array.
@@ -857,11 +898,11 @@ int igraph_vector_init_copy(igraph_vector_t *v, real_t *data, long int length) {
  * n is the size of the vector.
  */
 
-void igraph_vector_copy_to(const igraph_vector_t *v, real_t* to) {
+void igraph_vector_copy_to(const igraph_vector_t *v, igraph_real_t* to) {
   assert(v != NULL);
   assert(v->stor_begin != NULL);
   if (v->end != v->stor_begin) {
-    memcpy(to, v->stor_begin, sizeof(real_t) * (v->end - v->stor_begin));
+    memcpy(to, v->stor_begin, sizeof(igraph_real_t) * (v->end - v->stor_begin));
   }
 }
 
@@ -870,6 +911,7 @@ void igraph_vector_copy_to(const igraph_vector_t *v, real_t* to) {
  * \function igraph_vector_copy
  * \brief Initializes a vector from another vector object (constructor).
  * 
+ * </para><para>
  * The contents of the existing vector object will be copied to
  * the new one.
  * \param to Pointer to a not yet initialized vector object.
@@ -885,13 +927,13 @@ void igraph_vector_copy_to(const igraph_vector_t *v, real_t* to) {
 int igraph_vector_copy(igraph_vector_t *to, const igraph_vector_t *from) {
   assert(from != NULL);
   assert(from->stor_begin != NULL);
-  to->stor_begin=Calloc(igraph_vector_size(from), real_t);
+  to->stor_begin=Calloc(igraph_vector_size(from), igraph_real_t);
   if (to->stor_begin==0) {
     IGRAPH_ERROR("canot copy vector", IGRAPH_ENOMEM);
   }
   to->stor_end=to->stor_begin+igraph_vector_size(from);
   to->end=to->stor_end;
-  memcpy(to->stor_begin, from->stor_begin, igraph_vector_size(from)*sizeof(real_t));
+  memcpy(to->stor_begin, from->stor_begin, igraph_vector_size(from)*sizeof(igraph_real_t));
   
   return 0;
 }
@@ -901,18 +943,18 @@ int igraph_vector_copy(igraph_vector_t *to, const igraph_vector_t *from) {
  * \function igraph_vector_sum
  * \brief Calculates the sum of the elements in the vector.
  *
+ * </para><para>
  * For the empty vector 0.0 is returned.
  * \param v The vector object.
  * \return The sum of the elements.
  * 
- * 
  * Time complexity: O(n), the size of
  * the vector. 
  */
 
-real_t igraph_vector_sum(const igraph_vector_t *v) {
-  real_t res=0;
-  real_t *p;
+igraph_real_t igraph_vector_sum(const igraph_vector_t *v) {
+  igraph_real_t res=0;
+  igraph_real_t *p;
   assert(v != NULL);
   assert(v->stor_begin != NULL);
   for (p=v->stor_begin; p<v->end; p++) {
@@ -926,6 +968,7 @@ real_t igraph_vector_sum(const igraph_vector_t *v) {
  * \function igraph_vector_prod
  * \brief Calculates the product of the elements in the vector.
  * 
+ * </para><para>
  * For the empty vector one (1) is returned.
  * \param v The vector object.
  * \return The product of the elements.
@@ -934,9 +977,9 @@ real_t igraph_vector_sum(const igraph_vector_t *v) {
  * the vector. 
  */
 
-real_t igraph_vector_prod(const igraph_vector_t *v) {
-  real_t res=1;
-  real_t *p;
+igraph_real_t igraph_vector_prod(const igraph_vector_t *v) {
+  igraph_real_t res=1;
+  igraph_real_t *p;
   assert(v != NULL);
   assert(v->stor_begin != NULL);
   for (p=v->stor_begin; p<v->end; p++) {
@@ -950,6 +993,7 @@ real_t igraph_vector_prod(const igraph_vector_t *v) {
  * \function igraph_vector_init_seq
  * \brief Initializes a vector with a sequence.
  * 
+ * </para><para>
  * The vector will contain the numbers \p from,
  * \p from+1, ..., \p to.
  * \param v Pointer to an uninitialized vector object.
@@ -962,8 +1006,8 @@ real_t igraph_vector_prod(const igraph_vector_t *v) {
  * of elements in the vector. 
  */
 
-int igraph_vector_init_seq(igraph_vector_t *v, real_t from, real_t to) {
-  real_t *p;
+int igraph_vector_init_seq(igraph_vector_t *v, igraph_real_t from, igraph_real_t to) {
+  igraph_real_t *p;
   IGRAPH_CHECK(igraph_vector_init(v, to-from+1));
 
   for (p=v->stor_begin; p<v->end; p++) {
@@ -978,6 +1022,7 @@ int igraph_vector_init_seq(igraph_vector_t *v, real_t from, real_t to) {
  * \function igraph_vector_remove_section
  * \brief Deletes a section from a vector.
  * 
+ * </para><para>
  * Note that this function does not do range checking. The result is
  * undefined if you supply invalid limits.
  * \param v The vector object.
@@ -993,7 +1038,7 @@ void igraph_vector_remove_section(igraph_vector_t *v, long int from, long int to
   assert(v != NULL);
   assert(v->stor_begin != NULL);
   memmove(v->stor_begin+from, v->stor_begin+to,
-	  sizeof(real_t)*(v->end-v->stor_begin-to));
+	  sizeof(igraph_real_t)*(v->end-v->stor_begin-to));
   v->end -= (to-from);
 }
 
@@ -1022,6 +1067,7 @@ void igraph_vector_remove(igraph_vector_t *v, long int elem) {
  * \function igraph_vector_move_interval
  * \brief Copies a section of a vector.
  *
+ * </para><para>
  * The result of this function is undefined if the source and target
  * intervals overlap.
  * \param v The vector object.
@@ -1039,7 +1085,7 @@ int igraph_vector_move_interval(igraph_vector_t *v, long int begin, long int end
   assert(v != NULL);
   assert(v->stor_begin != NULL);
   memcpy(v->stor_begin+to, v->stor_begin+begin, 
-	 sizeof(real_t)*(end-begin));
+	 sizeof(igraph_real_t)*(end-begin));
 
   return 0;
 }
@@ -1094,8 +1140,8 @@ void igraph_vector_remove_negidx(igraph_vector_t *v, const igraph_vector_t *neg,
  * of elements in the vector.
  */
 
-bool_t igraph_vector_isininterval(const igraph_vector_t *v, real_t low, real_t high) {
-  real_t *ptr;
+igraph_bool_t igraph_vector_isininterval(const igraph_vector_t *v, igraph_real_t low, igraph_real_t high) {
+  igraph_real_t *ptr;
   assert(v != NULL);
   assert(v->stor_begin != NULL);
   for (ptr=v->stor_begin; ptr<v->end; ptr++) {
@@ -1121,8 +1167,8 @@ bool_t igraph_vector_isininterval(const igraph_vector_t *v, real_t low, real_t h
  * of elements in the vector.
  */
 
-bool_t igraph_vector_any_smaller(const igraph_vector_t *v, real_t limit) {
-  real_t *ptr;
+igraph_bool_t igraph_vector_any_smaller(const igraph_vector_t *v, igraph_real_t limit) {
+  igraph_real_t *ptr;
   assert(v != NULL);
   assert(v->stor_begin != NULL);
   for (ptr=v->stor_begin; ptr<v->end; ptr++) {
@@ -1148,7 +1194,7 @@ bool_t igraph_vector_any_smaller(const igraph_vector_t *v, real_t limit) {
  * of the vectors.
  */
 
-bool_t igraph_vector_is_equal(const igraph_vector_t *lhs, const igraph_vector_t *rhs) {
+igraph_bool_t igraph_vector_is_equal(const igraph_vector_t *lhs, const igraph_vector_t *rhs) {
   long int i, s;
   assert(lhs != 0);
   assert(rhs != 0);
@@ -1173,6 +1219,7 @@ bool_t igraph_vector_is_equal(const igraph_vector_t *lhs, const igraph_vector_t
  * \function igraph_vector_binsearch
  * \brief Finds an element by binary searching a sorted vector.
  * 
+ * </para><para>
  * It is assumed that the vector is sorted. If the specified element
  * (\p what) is not in the vector, then the
  * position of where it should be inserted (to keep the vector sorted)
@@ -1193,16 +1240,16 @@ bool_t igraph_vector_is_equal(const igraph_vector_t *lhs, const igraph_vector_t
  * \p v.
  */
 
-bool_t igraph_vector_binsearch(const igraph_vector_t *v, real_t what, long int *pos) {
+igraph_bool_t igraph_vector_binsearch(const igraph_vector_t *v, igraph_real_t what, long int *pos) {
   long int left=0;
   long int right=igraph_vector_size(v)-1;
 
-  while (left < right) {
+  while (left < right-1) {
     long int middle=(left+right)/2;
     if (VECTOR(*v)[middle] > what) {
-      right=middle-1;
+      right=middle;
     } else if (VECTOR(*v)[middle] < what) {
-      left=middle+1;
+      left=middle;
     } else {
       left=middle;
       break;
@@ -1213,8 +1260,81 @@ bool_t igraph_vector_binsearch(const igraph_vector_t *v, real_t what, long int *
     left=right;
   }
   
-  if (pos != 0 && VECTOR(*v)[left]==what) {
+  if (pos != 0) {
     *pos=left;
   }
   return VECTOR(*v)[left]==what;
 }
+
+/**
+ * \function igraph_vector_multiply
+ * \brief Multiply all elements of a vector by a constant
+ * 
+ * \param v The vector.
+ * \param by The constant.
+ * \return Error code. The current implementation always returns with success.
+ * 
+ * Added in version 0.2.</para><para>
+ * 
+ * Time complexity: O(n), the number of elements in a vector.
+ */
+
+void igraph_vector_multiply(igraph_vector_t *v, igraph_real_t by) {
+  long int i;
+  for (i=0; i<igraph_vector_size(v); i++) {
+    VECTOR(*v)[i] *= by;
+  }
+}
+
+igraph_bool_t igraph_vector_search(igraph_vector_t *v, long int from, igraph_real_t what, 
+			    long int *pos) {
+  long int i, n=igraph_vector_size(v);  
+  for (i=from; i<n; i++) {
+    if (VECTOR(*v)[i]==what) break;
+  }
+  
+  if (i<n) {
+    if (pos != 0) {
+      *pos=i;
+    }
+    return 1;
+  } else {
+    return 0;
+  }
+}
+
+int igraph_vector_filter_smaller(igraph_vector_t *v, igraph_real_t elem) {
+  long int i=0, n=igraph_vector_size(v);
+  long int s;
+  while (i<n && VECTOR(*v)[i]<elem) {
+    i++;
+  }
+  s=i;
+  
+  while (s<n && VECTOR(*v)[s]==elem) {
+    s++;
+  }
+  
+  igraph_vector_remove_section(v, 0, i+(s-i)/2);
+  return 0;
+}
+
+int igraph_vector_append(igraph_vector_t *to, const igraph_vector_t *from) {
+  long int tosize, fromsize;
+  
+  tosize=igraph_vector_size(to);
+  fromsize=igraph_vector_size(from);
+  IGRAPH_CHECK(igraph_vector_resize(to, tosize+fromsize));
+  memcpy(to->stor_begin+tosize, from->stor_begin, 
+	 sizeof(igraph_real_t)*fromsize);
+  to->end=to->stor_begin+tosize+fromsize;
+  
+  return 0;
+}
+
+int igraph_vector_get_interval(const igraph_vector_t *v, igraph_vector_t *res,
+			       long int from, long int to) {
+  IGRAPH_CHECK(igraph_vector_resize(res, to-from));
+  memcpy(res->stor_begin, v->stor_begin+from, (to-from)*sizeof(igraph_real_t));
+  return 0;
+}
